@@ -1,7 +1,6 @@
-from __future__ import unicode_literals
+from __future__ import division, print_function, unicode_literals
 
 import word2vec
-import numpy as np
 
 
 def load(fname, kind='auto', *args, **kwargs):
